@@ -1,12 +1,10 @@
-#Copyright barry king <barry@wyrdwright.com> and released under the GPL.
-#See http://www.gnu.org/licenses/gpl.html#TOC1 for details
 use 5.006;
 use strict;
 use warnings;
 no warnings qw(uninitialized);
 
 package Apache::Wyrd::Input::Condenser;
-our $VERSION = '0.93';
+our $VERSION = '0.94';
 use base qw(Apache::Wyrd::Input::Complex);
 use Apache::Wyrd::Services::SAK qw(token_parse);
 
@@ -148,7 +146,7 @@ sub _set_input {
 	my ($self, $input, $value) = @_;
 	return unless ($input->can('set'));
 	$input->set($value);
-	return undef;
+	return;
 }
 
 =pod
